--- /sys/amd64/amd64/busdma_machdep.c	Tue Mar 28 01:28:37 2006
+++ /sys/amd64/amd64/busdma_machdep.c	Thu Apr 27 11:55:55 2006
@@ -355,6 +355,107 @@
 }
 
 /*
+ * Allocate a vector of handles for mapping from kva/uva/physical
+ * address space into bus device space
+ */
+int
+bus_dmamap_create_v(bus_dma_tag_t dmat, int flags, bus_dmamap_t *mapp, size_t num)
+{
+	int error;
+
+	error = 0;
+
+	int i;
+
+	i = 0;
+
+	if (dmat->segments == NULL) {
+		dmat->segments = (bus_dma_segment_t *)malloc(
+		    sizeof(bus_dma_segment_t) * dmat->nsegments, M_DEVBUF,
+		    M_NOWAIT);
+		if (dmat->segments == NULL) {
+			CTR3(KTR_BUSDMA, "%s: tag %p error %d",
+			    __func__, dmat, ENOMEM);
+			return (ENOMEM);
+		}
+	}
+
+	/*
+	 * Bouncing might be required if the driver asks for an active
+	 * exclusion region, a data alignment that is stricter than 1, and/or
+	 * an active address boundary.
+	 */
+	if (dmat->flags & BUS_DMA_COULD_BOUNCE) {
+
+		/* Must bounce */
+		struct bounce_zone *bz;
+		int maxpages;
+
+		if (dmat->bounce_zone == NULL) {
+			if ((error = alloc_bounce_zone(dmat)) != 0)
+				return (error);
+		}
+		bz = dmat->bounce_zone;
+
+		*mapp = (bus_dmamap_t)malloc(sizeof(**mapp) * num, M_DEVBUF,
+					     M_NOWAIT | M_ZERO); 
+
+		if (*mapp == NULL) {
+			CTR3(KTR_BUSDMA, "%s: tag %p error %d",
+			    __func__, dmat, ENOMEM);
+			return (ENOMEM);
+		}
+
+		
+
+		/* Initialize the new map */
+
+		bus_dmamap_t *tempp = mapp;
+		
+		for(; i < num; i++) {
+			STAILQ_INIT(&((*mapp)->bpages));
+			(*mapp)++;
+		}
+
+		mapp = tempp;
+
+		/*
+		 * Attempt to add pages to our pool on a per-instance
+		 * basis up to a sane limit.
+		 */
+		if (dmat->alignment > 1)
+			maxpages = MAX_BPAGES;
+		else
+			maxpages = MIN(MAX_BPAGES, Maxmem -atop(dmat->lowaddr));
+		if ((dmat->flags & BUS_DMA_MIN_ALLOC_COMP) == 0
+		 || (dmat->map_count > 0 && bz->total_bpages < maxpages)) {
+			int pages;
+
+			pages = MAX(atop(dmat->maxsize), 1);
+			pages = MIN(maxpages - bz->total_bpages, pages);
+			pages = MAX(pages, 1);
+			if (alloc_bounce_pages(dmat, pages) < pages)
+				error = ENOMEM;
+
+			if ((dmat->flags & BUS_DMA_MIN_ALLOC_COMP) == 0) {
+				if (error == 0)
+					dmat->flags |= BUS_DMA_MIN_ALLOC_COMP;
+			} else {
+				error = 0;
+			}
+		}
+	} else {
+		*mapp = NULL;
+	}
+	if (error == 0)
+		dmat->map_count += num;
+	CTR4(KTR_BUSDMA, "%s: tag %p tag flags 0x%x error %d",
+	    __func__, dmat, dmat->flags, error);
+	return (error);
+}
+
+
+/*
  * Allocate a handle for mapping from kva/uva/physical
  * address space into bus device space.
  */
